////////////////////////////////////////////////////////////////////////////////
/**
 *  Bullet Physics によるベクトル，行列の計算例
 *
 */

#include <iostream>
#include "btBulletDynamicsCommon.h"

////////////////////////////////////////////////////////////////////////////////
/**
 *  行列 A を表示する
 */
void printMatrix(btMatrix3x3 A, const char *name){
  using namespace std;
  cout << name << " = \n"
       << "[ " << A[0].x() << " " << A[0].y() << " "  << A[0].z() << " ]\n"
       << "[ " << A[1].x() << " " << A[1].y() << " "  << A[1].z() << " ]\n"
       << "[ " << A[2].x() << " " << A[2].y() << " "  << A[2].z() << " ]" << endl;
}

////////////////////////////////////////////////////////////////////////////////
/**
 *  ベクトル v を表示する
 */
void printVector(btVector3 v, const char *name){
  using namespace std;
  cout << name << " = [ " << v.x() << " " << v.y() << " " << v.z() << " ]" << endl;
}

////////////////////////////////////////////////////////////////////////////////
/**
 *  main
 */
int main(){
  using namespace std;
  btVector3 b(1, 2, 3), c, x;
  btMatrix3x3 A(1, 0, 0,  0, 1, 0,  0, 0, 1), B, X;
  
  c.setValue(0.4, 0.5, 0.6);
  B.setValue(0.4, 0, 0,  0, 0.5, 0, 0, 0, 0.6);
  
  printMatrix(A, "A"); printMatrix(B, "B");
  printVector(b, "b"); printVector(c, "c");
  
  cout << "using btMatrix3x3..." << endl;
  x = A*b;
  printVector(x, "x = A*b");

  cout << "vector + vector" << endl;
  x = b + c;
  printVector(x, "x = b + c");

  cout << "matrix * matrix" << endl;
  X = A*B;
  printMatrix(X, "X = A * B");
  
  // A[0], A[1], etc. の型は btVector3 となるので，btVector3用の
  // 加算演算子 "+"で加算が行える
  cout << "matrix + matrix" << endl;
  X[0] = A[0] + B[0];  X[1] = A[1] + B[1];  X[2] = A[2] + B[2];
  printMatrix(X, "X = A + B");
  
  return 0;
}
