////////////////////////////////////////////////////////////////////////////////
/**
 *  球体の自由落下のシミュレーション
 *
 */

#include "BPWorld.h" // BPWorldを使うため
#include <cstdio>

btRigidBody *g_sphereBody; // 球体の力学データ

////////////////////////////////////////////////////////////////////////////////
/**
 *  シミュレーションのステップごとに呼ばれる関数（コールバック）
 *  物体の状態をモニタリングするために使う．
 */
void doMonitoring(btScalar currentTime){
  btVector3 pos = g_sphereBody->getCenterOfMassPosition(); // 質量中心位置を取得
  printf("y = %f\n", pos.y());
}

////////////////////////////////////////////////////////////////////////////////
/**
 *  物体を作成して，ワールドに配置する 
 */
void setUpObjects(BPWorld& bpWorld){
  btCollisionShape *shape = new btSphereShape(1); // 形状データの作成
  btTransform transform; // 座標変換データ 
  transform.setIdentity(); // 座標変換データの初期化（回転なし，平行移動なし）
  transform.setOrigin(btVector3(0, 10, 0)); // (X, Y, Z)の平行移動に設定する

  // 指定した形状の物体を作成し，座標変換データにより指定された
  // 位置・姿勢でワールドに配置する
  g_sphereBody = bpWorld.createRigidBody(1.0,       // 質量
                                         transform, // 初期位置，姿勢
                                         shape);    // 形状データ
  
  g_sphereBody->setRestitution(1.0); // 反発パラメータの設定
}

////////////////////////////////////////////////////////////////////////////////
/**
 *  プログラムの開始点
 */
int main(){
  // 動力学シミュレーションのワールドの準備
  BPWorld bpWorld("Falling Sphere", // ウィンドウのタイトル
                  640, 480,         // ウィンドウの幅と高さ [pixels]
                  doMonitoring);    // モニタリング用関数へのポインタ
  
  setUpObjects(bpWorld); // 物体を作成して配置する
  bpWorld.enableFixedTimeStep(true);
  bpWorld.setTimeStep(0.01); // seconds
  bpWorld.setGlobalERP(0.2); // ERP
  bpWorld.start(); // ウィンドウを表示して，シミュレーションを開始する
  
  return 0;
}
