////////////////////////////////////////////////////////////////////////////////
/**
 *  Bullet Physics の使用例
 *
 *  http://bulletphysics.org/mediawiki-1.5.8/index.php/Hello_World に掲載の
 *  ソースコードを基に作成
 */

#include <iostream>
#include <fstream>
#include <iomanip>
#include <btBulletDynamicsCommon.h>

int main(void)
{
  using namespace std;
  btBroadphaseInterface* broadphase = new btDbvtBroadphase();
  btDefaultCollisionConfiguration* collisionConfiguration = new btDefaultCollisionConfiguration();
  btCollisionDispatcher* dispatcher = new btCollisionDispatcher(collisionConfiguration);
  btSequentialImpulseConstraintSolver* solver = new btSequentialImpulseConstraintSolver;
  btDiscreteDynamicsWorld* dynamicsWorld = new btDiscreteDynamicsWorld(dispatcher,broadphase,solver,collisionConfiguration);
 
  dynamicsWorld->setGravity(btVector3(0,-10,0));
 
  btCollisionShape* groundShape = new btStaticPlaneShape(btVector3(0,1,0),1);
  btCollisionShape* fallShape = new btSphereShape(1);
 
 
  btDefaultMotionState* groundMotionState = new btDefaultMotionState(btTransform(btQuaternion(0,0,0,1),btVector3(0,-1,0)));
  btRigidBody::btRigidBodyConstructionInfo
    groundRigidBodyCI(0,groundMotionState,groundShape,btVector3(0,0,0));
  btRigidBody* groundRigidBody = new btRigidBody(groundRigidBodyCI);
  dynamicsWorld->addRigidBody(groundRigidBody);
 
 
  btDefaultMotionState* fallMotionState =
    new btDefaultMotionState(btTransform(btQuaternion(0,0,0,1),btVector3(0,50,0)));
  btScalar mass = 1;
  btVector3 fallInertia(0,0,0);
  fallShape->calculateLocalInertia(mass,fallInertia);
  btRigidBody::btRigidBodyConstructionInfo fallRigidBodyCI(mass,fallMotionState,fallShape,fallInertia);
  btRigidBody* fallRigidBody = new btRigidBody(fallRigidBodyCI);
  dynamicsWorld->addRigidBody(fallRigidBody);
 
  btScalar t = 0;
  btScalar dt = 1/60.0;
  int maxSubSteps = 10;
  ofstream fout("result.txt");
  fout << scientific // "0.000000e+00"の形式での出力を指定する
       << setprecision(7); // 有効桁数．32-bit floatの仮数部幅 = 24 bits -> log10(2^24) = 7.224...
  for(int i = 0; i < 300; i++){
    btTransform trans;
    fallRigidBody->getMotionState()->getWorldTransform(trans);
    fout << t << " " << trans.getOrigin().getY() << endl;
    cout << t << " " << trans.getOrigin().getY() << endl;
    dynamicsWorld->stepSimulation(dt, maxSubSteps); 
    t += dt;
  }
 
  dynamicsWorld->removeRigidBody(fallRigidBody);
  delete fallRigidBody->getMotionState();
  delete fallRigidBody;
  dynamicsWorld->removeRigidBody(groundRigidBody);
  delete groundRigidBody->getMotionState();
  delete groundRigidBody;
  delete fallShape;
  delete groundShape;
  delete dynamicsWorld;
  delete solver;
  delete collisionConfiguration;
  delete dispatcher;
  delete broadphase;
 
  return 0;
}
