////////////////////////////////////////////////////////////////////////////////
/**
 *  箱を投げるシミュレーション
 *
 */

#include "BPWorld.h" // BPWorldを使うため
#include <cstdio> // C言語のstdio.hを使うときのC++式の書き方

#define TEST_EULER_YPR // getEulerYPR() を試す場合
//#define TEST_EULER_ZYX // getEulerZYX() を試す場合

//#define TEST_X_AXIS_ROT // X軸まわりに回す場合
//#define TEST_Y_AXIS_ROT // Y軸まわりに回す場合
#define TEST_Z_AXIS_ROT // Z軸まわりに回す場合

btRigidBody *g_box; // 箱の力学データ

////////////////////////////////////////////////////////////////////////////////
/**
 *  シミュレーションのステップごとに呼ばれる関数（コールバック）
 *  物体の状態をモニタリングするために使う．
 */
void doMonitoring(btScalar currentTime){
  btScalar yaw, pitch, roll;

#ifdef TEST_EULER_YPR // btMatrix3x3::getEulerYPR(rotY, rotX, rotZ)
  btMatrix3x3(g_box->getOrientation()).getEulerYPR(yaw, pitch, roll);
#elif defined TEST_EULER_ZYX // btMatrix3x3::getEulerZYX(rotZ, rotY, rotX)
  btMatrix3x3(g_box->getOrientation()).getEulerZYX(yaw, pitch, roll);
#endif
  printf("[yaw, pitch, roll] = [ %7.3f, %7.3f, %7.3f ]\n",
         btDegrees(yaw), btDegrees(pitch), btDegrees(roll));
}

////////////////////////////////////////////////////////////////////////////////
/**
 *  物体を作成して，ワールドに配置する
 */
void setUpObjects(BPWorld& bpWorld){
  btCollisionShape* shape = new btBoxShape(btVector3(0.5, 2, 1));
  btTransform transform;
  transform.setIdentity();
  transform.setOrigin(btVector3(5, 10, 0)); // 初期位置 (X, Y, Z)
  btQuaternion quat;

  // btQuaternion::setEuler(rotY, rotX, rotZ) による回転の指定
  // X, Y, Z の順でなく Y, X, Z の順
#ifdef TEST_Y_AXIS_ROT
  quat.setEuler(btRadians(30), 0, 0); // 初期姿勢, Y軸まわりの回転のみ
#elif defined TEST_X_AXIS_ROT
  quat.setEuler(0, btRadians(30), 0); // 初期姿勢, X軸まわりの回転のみ
#elif defined TEST_Z_AXIS_ROT
  quat.setEuler(0, 0, btRadians(30)); // 初期姿勢, Z軸まわりの回転のみ
#endif

  transform.setRotation(quat);

  g_box = bpWorld.createRigidBody(1.0, // 質量
                                  transform,
                                  shape);

  g_box->setRestitution(0.8);
  g_box->setLinearVelocity(btVector3(-10, 0, 0)); // 初期速度の設定
}

////////////////////////////////////////////////////////////////////////////////
/**
 *  プログラムの開始点
 */
int main(){
  // 動力学シミュレーションのワールドの準備
  BPWorld bpWorld("Thrown Box", // ウィンドウのタイトル
                  640, 480,         // ウィンドウの幅と高さ [pixels]
                  doMonitoring);    // モニタリング用関数へのポインタ

  setUpObjects(bpWorld); // 物体を作成して配置する
  bpWorld.start(); // ウィンドウを表示して，シミュレーションを開始する

  return 0;
}
